/*
    Digital circuits emulation library

    Copyright (c) 2004 Alessandro Scotti
    http://www.ascotti.org/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef DCE_7427_H_
#define DCE_7427_H_

/*
    Triple 3-input positive NOR gate.
*/
class D7427 : public DChip
{
public:
    D7427() : DChip(14) {
        //  1: 1A -> input for port #1
        //  2: 1B -> input for port #1
        //  3: 2A -> input for port #2
        //  4: 2B -> input for port #2
        //  5: 2C -> input for port #2
        connectLine(  6, y2_ ); // 2Y, output of port #2
        connectLine(  7, DLine::Ground );
        connectLine(  8, y3_ ); // 3Y, output of port #3
        //  9: 3A -> input for port #3
        // 10: 3B -> input for port #3
        // 11: 3C -> input for port #3
        connectLine( 12, y1_ ); // 1Y, output of port #1
        // 13: 1C -> input for port #1
        connectLine( 14, DLine::Power );

        y1_ = 0;
        y2_ = 0;
        y3_ = 0;
    }

    void update() {
        y1_ = 1 ^ (line( 1) | line( 2) | line(13));
        y2_ = 1 ^ (line( 3) | line( 4) | line( 5));
        y3_ = 1 ^ (line( 9) | line(10) | line(11));
    }

private:
    DBit    y1_;
    DBit    y2_;
    DBit    y3_;
};

#endif // DCE_7427_H_
