/*
    Digital circuits emulation library

    Copyright (c) 2004 Alessandro Scotti
    http://www.ascotti.org/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef DCE_9322_H_
#define DCE_9322_H_

/*
    Quadruple 2-line to 1-line data selector/multiplexer.
*/
class D9322 : public DChip
{
public:
    D9322() : DChip(16) {
        //  1: SELECT
        //  2: A1 -> input for port #1
        //  3: B1 -> input for port #1
        connectLine(  4, y1_ ); // 1Y, output of port #1
        //  5: A2 -> input for port #2
        //  6: B2 -> input for port #2
        connectLine(  7, y2_ ); // 2Y, output of port #2
        connectLine(  8, DLine::Ground );
        connectLine(  9, y3_ ); // 3Y, output of port #3
        // 10: B3 -> input for port #3
        // 11: A3 -> input for port #3
        connectLine( 12, y4_ ); // 4Y, output of port #4
        // 13: B4 -> input for port #4
        // 14: A4 -> input for port #4
        // 15: STROBE
        connectLine( 16, DLine::Power );

        y1_ = 0;
        y2_ = 0;
        y3_ = 0;
        y4_ = 0;
    }

    void update() {
        if( line(15) ) {
            // All outputs low on strobe
            y1_ = 0;
            y2_ = 0;
            y3_ = 0;
            y4_ = 0;
        }
        else {
            DBit    select = line( 1);

            y1_ = select ? line( 3) : line( 2);
            y2_ = select ? line( 6) : line( 5);
            y3_ = select ? line(10) : line(11);
            y4_ = select ? line(13) : line(14);
        }
    }

private:
    DBit    y1_;
    DBit    y2_;
    DBit    y3_;
    DBit    y4_;
};

#endif // DCE_9322_H_
